#include "gtbmouse.h"


#ifdef OUTLINE
#define inline
#include <gtb/graphics/box3.ipp>
#undef inline
#endif

using namespace std;


BEGIN_GTBMOUSENS


// see AT&T notes, p. 34
const unsigned Box3::_vertex_indices[6][4] = {
	{0, 1, 3, 2},
	{4, 6, 7, 5},
	{0, 4, 5, 1},
	{2, 3, 7, 6},
	{0, 2, 6, 4},
	{1, 5, 7, 3}
};



real_t Box3::shortest_axis_length() const
{
	real_t dx = x_length();
	real_t dy = y_length();
	real_t dz = z_length();
	return min(min(dx, dy), dz);
}


real_t Box3::longest_axis_length() const
{
	real_t dx = x_length();
	real_t dy = y_length();
	real_t dz = z_length();
	return max(max(dx, dy), dz);
}


int Box3::classify_position(const Box3 &b) const
{
	if ((_max_pt[0] < b._min_pt[0]) || (_min_pt[0] > b._max_pt[0]) ||
	    (_max_pt[1] < b._min_pt[1]) || (_min_pt[1] > b._max_pt[1]) ||
	    (_max_pt[2] < b._min_pt[2]) || (_min_pt[2] > b._max_pt[2])) {
		return OUTSIDE;
	}

	if ((_min_pt[0] <= b._min_pt[0]) && (_max_pt[0] >= b._max_pt[0]) &&
	    (_min_pt[1] <= b._min_pt[1]) && (_max_pt[1] >= b._max_pt[1]) &&
	    (_min_pt[2] <= b._min_pt[2]) && (_max_pt[2] >= b._max_pt[2])) {
		return INSIDE;
        }

	return INTERSECT;
}

#if 0
Polygon3 Box3::face(unsigned i) const
{
	assert(i < 6);
	return Polygon3(vertex(i, 0),
			vertex(i, 1),
			vertex(i, 2),
			vertex(i, 3),
			normal(i));
}
#endif

Plane Box3::plane(unsigned i) const
{
	assert(i < 6);
	return Plane(vertex(_vertex_indices[i][0]),
		     vertex(_vertex_indices[i][1]),
		     vertex(_vertex_indices[i][2]));
}


void Box3::render() const
{
	glBegin(GL_QUADS);
	for (unsigned i = 0; i < 6; i++) {
		normal(i).load_as_normal();
		vertex(i, 0).load();
		vertex(i, 1).load();
		vertex(i, 2).load();
		vertex(i, 3).load();
	}
	glEnd();
}


void Box3::outline() const
{
	glPushAttrib(GL_ENABLE_BIT);
	glDisable(GL_LIGHTING);

	glBegin(GL_LINE_LOOP);
	glVertex3f(_min_pt[0], _min_pt[1], _min_pt[2]);
	glVertex3f(_max_pt[0], _min_pt[1], _min_pt[2]);
	glVertex3f(_max_pt[0], _max_pt[1], _min_pt[2]);
	glVertex3f(_min_pt[0], _max_pt[1], _min_pt[2]);
	glEnd();

	glBegin(GL_LINE_LOOP);
	glVertex3f(_min_pt[0], _min_pt[1], _max_pt[2]);
	glVertex3f(_max_pt[0], _min_pt[1], _max_pt[2]);
	glVertex3f(_max_pt[0], _max_pt[1], _max_pt[2]);
	glVertex3f(_min_pt[0], _max_pt[1], _max_pt[2]);
	glEnd();

	glBegin(GL_LINES);
	glVertex3f(_min_pt[0], _min_pt[1], _min_pt[2]);
	glVertex3f(_min_pt[0], _min_pt[1], _max_pt[2]);
	glVertex3f(_max_pt[0], _min_pt[1], _min_pt[2]);
	glVertex3f(_max_pt[0], _min_pt[1], _max_pt[2]);
	glVertex3f(_max_pt[0], _max_pt[1], _min_pt[2]);
	glVertex3f(_max_pt[0], _max_pt[1], _max_pt[2]);
	glVertex3f(_min_pt[0], _max_pt[1], _min_pt[2]);
	glVertex3f(_min_pt[0], _max_pt[1], _max_pt[2]);
	glEnd();

	glPopAttrib();
}


Box3 Box3::bounding_box(const vector<Point3> &v)
{
	assert(v.size() > 0);
	Point3 p = v[0];
	real_t min_x = p.x(), min_y = p.y(), min_z = p.z();
	real_t max_x = p.x(), max_y = p.y(), max_z = p.z();
	for (unsigned i = 1; i < v.size(); i++) {
		p = v[i];
		min_x = min(min_x, p.x());
		min_y = min(min_y, p.y());
		min_z = min(min_z, p.z());
		max_x = max(max_x, p.x());
		max_y = max(max_y, p.y());
		max_z = max(max_z, p.z());
	}
	return Box3(min_x, min_y, min_z,
		    max_x, max_y, max_z);
}


Box3 Box3::make_union(const Box3 &b1, const Box3 &b2)
{
	return Box3(min(b1.x_min(), b2.x_min()),
		    min(b1.y_min(), b2.y_min()),
		    min(b1.z_min(), b2.z_min()),
		    max(b1.x_max(), b2.x_max()),
		    max(b1.y_max(), b2.y_max()),
		    max(b1.z_max(), b2.z_max()));
}


/* Copyright (c) Hewlett Packard Company, 1997.  */

/* This program is freely distributable without licensing fees 
   and is provided without guarantee or warrantee expressed or 
   implied. This program is -not- in the public domain. */

#define GL_OCCLUSION_TEST_HP			0x8165
#define GL_OCCLUSION_RESULT_HP			0x8166
#define GL_OCCLUSION_TEST_HP_OLD		0x816E
#define GL_OCCLUSION_RESULT_HP_OLD		0x816F


bool Box3::is_visible() const
{
#ifndef HP_EXT
	return true;
#else
	GLfloat x1 = _min_pt[0],
		y1 = _min_pt[1],
		z1 = _min_pt[2],
		x2 = _max_pt[0],
		y2 = _max_pt[1],
		z2 = _max_pt[2];
	GLboolean   result;

	glDisable(GL_STENCIL_TEST);
	glEnable(GL_OCCLUSION_TEST_HP);
	glDepthMask(GL_FALSE);
	glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);

	glBegin(GL_QUADS);

	/* Front */
	glVertex3f(x1,y1,z1);
	glVertex3f(x1,y2,z1);
	glVertex3f(x2,y2,z1);
	glVertex3f(x2,y1,z1);

	/* Right */
	glVertex3f(x2,y2,z1);
	glVertex3f(x2,y1,z1);
	glVertex3f(x2,y1,z2);
	glVertex3f(x2,y2,z2);

	/* Back */
	glVertex3f(x2,y2,z2);
	glVertex3f(x2,y1,z2);
	glVertex3f(x1,y1,z2);
	glVertex3f(x1,y2,z2);

	/* Left */
	glVertex3f(x1,y2,z2);
	glVertex3f(x1,y1,z2);
	glVertex3f(x1,y1,z1);
	glVertex3f(x1,y2,z1);

	/* Top */
	glVertex3f(x2,y2,z1);
	glVertex3f(x2,y2,z2);
	glVertex3f(x1,y2,z2);
	glVertex3f(x1,y2,z1);

	/* Bottom */
	glVertex3f(x2,y1,z1);
	glVertex3f(x2,y1,z2);
	glVertex3f(x1,y1,z2);
	glVertex3f(x1,y1,z1);
	glEnd();

	glGetBooleanv(GL_OCCLUSION_RESULT_HP, &result);
	glDisable(GL_OCCLUSION_TEST_HP);
	glEnable(GL_LIGHTING);
	glDepthMask(GL_TRUE);
	glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
	glEnable(GL_STENCIL_TEST);
	fprintf(stderr, "visible: %d\n", result);
	return (result == GL_TRUE);
#endif
}


END_GTBMOUSENS
