#include "gtbmouse.h"
#ifndef WIN32
#include <gtb/graphics/coordinate_system.hpp>
#include <gtb/graphics/line3.hpp>
#include <gtb/graphics/color_rgb.hpp>
#include <GL/gl.h>
#endif // WIN32


#ifdef OUTLINE
#define inline
#include <gtb/graphics/coordinate_system.ipp>
#undef inline
#endif


BEGIN_GTBMOUSENS


CoordinateSystem::CoordinateSystem()
	: m_origin(POINT3_ZERO),
	  m_x(VECTOR3_POSITIVE_X),
	  m_y(VECTOR3_POSITIVE_Y),
	  m_z(VECTOR3_POSITIVE_Z)
{
}


CoordinateSystem::CoordinateSystem(const CoordinateSystem &cs)
	: m_origin(cs.m_origin),
	  m_x(cs.m_x),
	  m_y(cs.m_y),
	  m_z(cs.m_z)
{
}


CoordinateSystem::CoordinateSystem(const Point3 &a_origin,
				   const Vector3 &a_x,
				   const Vector3 &a_y,
				   const Vector3 &a_z)
	: m_origin(a_origin),
	  m_x(a_x),
	  m_y(a_y),
	  m_z(a_z)
{
}


CoordinateSystem &CoordinateSystem::operator=(const CoordinateSystem &cs)
{
	m_origin = cs.m_origin;
	m_x = cs.m_x;
	m_y = cs.m_y;
	m_z = cs.m_z;
	return *this;
}


Matrix4 CoordinateSystem::matrix() const
{
	return Matrix4(m_x.x(), m_y.x(), m_z.x(), m_origin.x(),
		       m_x.y(), m_y.y(), m_z.y(), m_origin.y(),
		       m_x.z(), m_y.z(), m_z.z(), m_origin.z(),
		       0.0, 0.0, 0.0, 1.0);
}


Matrix4 CoordinateSystem::inverse_matrix() const
{
	Matrix4 ir(m_x.x(), m_x.y(), m_x.z(), 0.0,
		   m_y.x(), m_y.y(), m_y.z(), 0.0,
		   m_z.x(), m_z.y(), m_z.z(), 0.0,
		   0.0, 0.0, 0.0, 1.0);

	Matrix4 it(1.0, 0.0, 0.0, -m_origin.x(),
		   0.0, 1.0, 0.0, -m_origin.y(),
		   0.0, 0.0, 1.0, -m_origin.z(),
		   0.0, 0.0, 0.0, 1.0);

	return ir * it;
}


CoordinateSystem &CoordinateSystem::rotate(const Line3 &l, real_t theta)
{
	m_origin.rotate(l, theta);
	m_x.rotate(l.direction(), theta);
	m_y.rotate(l.direction(), theta);
	m_z.rotate(l.direction(), theta);
	return *this;
}


CoordinateSystem &CoordinateSystem::translate(const Vector3 &t)
{
	m_origin.translate(t);
	return *this;
}


CoordinateSystem &CoordinateSystem::move_to(const Point3 &p)
{
	m_origin = p;
	return *this;
}


void CoordinateSystem::render(real_t length) const
{
	glPushAttrib(GL_ENABLE_BIT);
	glDisable(GL_LIGHTING);

	const Point3 &o = m_origin;
	Point3 px = o + length * m_x;
	Point3 py = o + length * m_y;
	Point3 pz = o + length * m_z;

#if 0
	COLOR_RGB_RED.load();
	glBegin(GL_LINES);
	glVertex3f(o.x(), o.y(), o.z());
	glVertex3f(px.x(), px.y(), px.z());
	glEnd();

	COLOR_RGB_GREEN.load();
	glBegin(GL_LINES);
	glVertex3f(o.x(), o.y(), o.z());
	glVertex3f(py.x(), py.y(), py.z());
	glEnd();

	COLOR_RGB_BLUE.load();
	glBegin(GL_LINES);
	glVertex3f(o.x(), o.y(), o.z());
	glVertex3f(pz.x(), pz.y(), pz.z());
	glEnd();
#endif

	glPopAttrib();

//  	fprintf(stderr, "o=%f %f %f\n", o.x(), o.y(), o.z());
//  	fprintf(stderr, "x=%f %f %f\n", px.x(), px.y(), px.z());
//  	fprintf(stderr, "y=%f %f %f\n", py.x(), py.y(), py.z());
//  	fprintf(stderr, "z=%f %f %f\n", pz.x(), pz.y(), pz.z());
}


END_GTBMOUSENS
