#ifndef GTB_COORDINATE_SYSTEM_INCLUDED
#define GTB_COORDINATE_SYSTEM_INCLUDED

#include "gtbmousecommon.h"
#include "point3.hpp"
#include "vector3.hpp"

BEGIN_GTBMOUSENS


class CoordinateSystem {
public:
	CoordinateSystem();
	CoordinateSystem(const CoordinateSystem &cs);
	CoordinateSystem(const Point3 &origin,
			 const Vector3 &x,
			 const Vector3 &y,
			 const Vector3 &z);
	CoordinateSystem &operator=(const CoordinateSystem &cs);

	bool operator==(const CoordinateSystem &cs);
	bool operator!=(const CoordinateSystem &cs);

	const Point3 &origin() const;
	const Vector3 &x() const;
	const Vector3 &y() const;
	const Vector3 &z() const;

	CoordinateSystem &reset(const Point3 &origin,
				const Vector3 &x,
				const Vector3 &y,
				const Vector3 &z);

	Matrix4 matrix() const;
	Matrix4 inverse_matrix() const;

	CoordinateSystem &rotate(const Line3 &l, real_t theta);
	CoordinateSystem &translate(const Vector3 &t);
	CoordinateSystem &move_to(const Point3 &p);

	void render(real_t length = 1.0) const;

protected:
	Point3 m_origin;
	Vector3 m_x;
	Vector3 m_y;
	Vector3 m_z;
};


END_GTBMOUSENS

#ifndef OUTLINE
#include "coordinate_system.ipp"
#endif

#endif // GTB_COORDINATE_SYSTEM_INCLUDED
