#include "gtbmouse.h"
#ifndef WIN32
#include <gtb/math/matrix4.hpp>
#include <gtb/error/error.hpp>
#include <gtb/math/math.hpp>
#include <math.h>
#include <stdlib.h>
#endif // WIN32

#ifdef OUTLINE
#define inline
#include <gtb/math/matrix4.ipp>
#undef inline
#endif

using namespace std;

BEGIN_GTBMOUSENS


const Matrix4 MATRIX4_ZERO(0.0, 0.0, 0.0, 0.0,
			   0.0, 0.0, 0.0, 0.0,
			   0.0, 0.0, 0.0, 0.0,
			   0.0, 0.0, 0.0, 0.0);


const Matrix4 MATRIX4_IDENTITY(1.0, 0.0, 0.0, 0.0,
			       0.0, 1.0, 0.0, 0.0,
			       0.0, 0.0, 1.0, 0.0,
			       0.0, 0.0, 0.0, 1.0);


Matrix4::Matrix4()
{
	_m[0][0] = 0.0;
	_m[0][1] = 0.0;
	_m[0][2] = 0.0;
	_m[0][3] = 0.0;

	_m[1][0] = 0.0;
	_m[1][1] = 0.0;
	_m[1][2] = 0.0;
	_m[1][3] = 0.0;

	_m[2][0] = 0.0;
	_m[2][1] = 0.0;
	_m[2][2] = 0.0;
	_m[2][3] = 0.0;

	_m[3][0] = 0.0;
	_m[3][1] = 0.0;
	_m[3][2] = 0.0;
	_m[3][3] = 0.0;
}


Matrix4::Matrix4(float m[4][4])
{
	_m[0][0] = m[0][0];
	_m[0][1] = m[0][1];
	_m[0][2] = m[0][2];
	_m[0][3] = m[0][3];

	_m[1][0] = m[1][0];
	_m[1][1] = m[1][1];
	_m[1][2] = m[1][2];
	_m[1][3] = m[1][3];

	_m[2][0] = m[2][0];
	_m[2][1] = m[2][1];
	_m[2][2] = m[2][2];
	_m[2][3] = m[2][3];

	_m[3][0] = m[3][0];
	_m[3][1] = m[3][1];
	_m[3][2] = m[3][2];
	_m[3][3] = m[3][3];
}


Matrix4::Matrix4(float m[16])
{
	_m[0][0] = m[0];
	_m[0][1] = m[1];
	_m[0][2] = m[2];
	_m[0][3] = m[3];

	_m[1][0] = m[4];
	_m[1][1] = m[5];
	_m[1][2] = m[6];
	_m[1][3] = m[7];

	_m[2][0] = m[8];
	_m[2][1] = m[9];
	_m[2][2] = m[10];
	_m[2][3] = m[11];

	_m[3][0] = m[12];
	_m[3][1] = m[13];
	_m[3][2] = m[14];
	_m[3][3] = m[15];
}


Matrix4::Matrix4(double m[4][4])
{
	_m[0][0] = m[0][0];
	_m[0][1] = m[0][1];
	_m[0][2] = m[0][2];
	_m[0][3] = m[0][3];

	_m[1][0] = m[1][0];
	_m[1][1] = m[1][1];
	_m[1][2] = m[1][2];
	_m[1][3] = m[1][3];

	_m[2][0] = m[2][0];
	_m[2][1] = m[2][1];
	_m[2][2] = m[2][2];
	_m[2][3] = m[2][3];

	_m[3][0] = m[3][0];
	_m[3][1] = m[3][1];
	_m[3][2] = m[3][2];
	_m[3][3] = m[3][3];
}


Matrix4::Matrix4(double m[16])
{
	_m[0][0] = m[0];
	_m[0][1] = m[1];
	_m[0][2] = m[2];
	_m[0][3] = m[3];

	_m[1][0] = m[4];
	_m[1][1] = m[5];
	_m[1][2] = m[6];
	_m[1][3] = m[7];

	_m[2][0] = m[8];
	_m[2][1] = m[9];
	_m[2][2] = m[10];
	_m[2][3] = m[11];

	_m[3][0] = m[12];
	_m[3][1] = m[13];
	_m[3][2] = m[14];
	_m[3][3] = m[15];
}


Matrix4::Matrix4(real_t m00, real_t m01, real_t m02, real_t m03,
		 real_t m10, real_t m11, real_t m12, real_t m13,
		 real_t m20, real_t m21, real_t m22, real_t m23,
		 real_t m30, real_t m31, real_t m32, real_t m33)
{
	_m[0][0] = m00;
	_m[0][1] = m01;
	_m[0][2] = m02;
	_m[0][3] = m03;

	_m[1][0] = m10;
	_m[1][1] = m11;
	_m[1][2] = m12;
	_m[1][3] = m13;

	_m[2][0] = m20;
	_m[2][1] = m21;
	_m[2][2] = m22;
	_m[2][3] = m23;

	_m[3][0] = m30;
	_m[3][1] = m31;
	_m[3][2] = m32;
	_m[3][3] = m33;
}


Matrix4::Matrix4(const Matrix4 &m)
{
	_m[0][0] = m._m[0][0];
	_m[0][1] = m._m[0][1];
	_m[0][2] = m._m[0][2];
	_m[0][3] = m._m[0][3];

	_m[1][0] = m._m[1][0];
	_m[1][1] = m._m[1][1];
	_m[1][2] = m._m[1][2];
	_m[1][3] = m._m[1][3];

	_m[2][0] = m._m[2][0];
	_m[2][1] = m._m[2][1];
	_m[2][2] = m._m[2][2];
	_m[2][3] = m._m[2][3];

	_m[3][0] = m._m[3][0];
	_m[3][1] = m._m[3][1];
	_m[3][2] = m._m[3][2];
	_m[3][3] = m._m[3][3];
}


Matrix4 &Matrix4::operator=(const Matrix4 &m)
{
	if (&m != this) {
		_m[0][0] = m._m[0][0];
		_m[0][1] = m._m[0][1];
		_m[0][2] = m._m[0][2];
		_m[0][3] = m._m[0][3];

		_m[1][0] = m._m[1][0];
		_m[1][1] = m._m[1][1];
		_m[1][2] = m._m[1][2];
		_m[1][3] = m._m[1][3];

		_m[2][0] = m._m[2][0];
		_m[2][1] = m._m[2][1];
		_m[2][2] = m._m[2][2];
		_m[2][3] = m._m[2][3];

		_m[3][0] = m._m[3][0];
		_m[3][1] = m._m[3][1];
		_m[3][2] = m._m[3][2];
		_m[3][3] = m._m[3][3];
	}
	return *this;
}


Matrix4 &Matrix4::make_identity()
{
	_m[0][0] = 1.0;
	_m[0][1] = 0.0;
	_m[0][2] = 0.0;
	_m[0][3] = 0.0;

	_m[1][0] = 0.0;
	_m[1][1] = 1.0;
	_m[1][2] = 0.0;
	_m[1][3] = 0.0;

	_m[2][0] = 0.0;
	_m[2][1] = 0.0;
	_m[2][2] = 1.0;
	_m[2][3] = 0.0;

	_m[3][0] = 0.0;
	_m[3][1] = 0.0;
	_m[3][2] = 0.0;
	_m[3][3] = 1.0;

	return *this;
}


Matrix4 &Matrix4::negate()
{
	_m[0][0] = -_m[0][0];
	_m[0][1] = -_m[0][1];
	_m[0][2] = -_m[0][2];
	_m[0][3] = -_m[0][3];

	_m[1][0] = -_m[1][0];
	_m[1][1] = -_m[1][1];
	_m[1][2] = -_m[1][2];
	_m[1][3] = -_m[1][3];

	_m[2][0] = -_m[2][0];
	_m[2][1] = -_m[2][1];
	_m[2][2] = -_m[2][2];
	_m[2][3] = -_m[2][3];

	_m[3][0] = -_m[3][0];
	_m[3][1] = -_m[3][1];
	_m[3][2] = -_m[3][2];
	_m[3][3] = -_m[3][3];

	return *this;
}


bool Matrix4::LU_decomposition(int p[4])
{
	int i, j, k;

	for (j = 0; j < 3; j++) {

		// Find line of pivot
		p[j] = j;
		for (i = j + 1; i < 4; i++) {
			if (fabs(_m[i][j]) > fabs(_m[p[j]][j])) {
				p[j] = i;
			}
		}

		// Swap lines if necessary
		if (p[j] != j) {
			i = p[j];
			for (k = 0; k < 4; k++) {
				real_t t = _m[i][k];
				_m[i][k] = _m[j][k];
				_m[j][k] = t;
			}
		}

		// Eliminate elements below diagonal
		for (i = j + 1; i < 4; i++) {
			_m[i][j] = -_m[i][j] / _m[j][j];
			for (k = j + 1; k < 4; k++) {
				_m[i][k] += _m[i][j] * _m[j][k];
			}
		}
	}
	return true;
}


void Matrix4::LU_back_substitution(const int p[4],
				   Matrix4 &b,
				   Matrix4 &x)
{
	int i, j, k;

	// Swap lines of b when necessary
	for (i = 0; i < 3; i++) {
		if (p[i] != i) {
			for (j = 0, k = p[i]; j < 4; j++) {
				real_t t = b._m[i][j];
				b._m[i][j] = b._m[k][j];
				b._m[k][j] = t;
			}
		}
	}

	// Apply multipliers to b
	for (j = 0; j < 3 ; j++) {
		for (i = j + 1; i < 4; i++) {
			for (k = 0; k < 4; k++) {
				b._m[i][k] += _m[i][j] * b._m[j][k];
			}
		}
	}

	// Back substitution
	for (k = 0; k < 4; k++) {
		for (i = 3; i >= 0; i--) {
			x._m[i][k] = b._m[i][k];
			for (j = i + 1; j < 4; j++) {
				x._m[i][k] -= _m[i][j] * x._m[j][k];
			}
//			assert(!real::is_zero(_m[i][i]));
			x._m[i][k] /= _m[i][i];
		}
	}
}


Matrix4 Matrix4::inverse() const
{
	Matrix4 a(*this);
	Matrix4 b(MATRIX4_IDENTITY);
	Matrix4 x;
	int p[4];

	a.LU_decomposition(p);
	a.LU_back_substitution(p, b, x);
	return x;
}


real_t Matrix4::det() const
{
	real_t d;
	Matrix4 t(*this);
	int p[4];

	t.LU_decomposition(p);
	d = 1.0;
	for (int i = 0; i < 3; i++) {
		d *= t._m[i][i];
		if (p[i] != i) {
			d = -d;
		}
	}
	d *= t._m[3][3];
	return d;
}


void Matrix4::print(FILE *fp) const
{
//	REQUIRE(NULL != fp);
	for (unsigned i = 0; i < 4; i++) {
		for (unsigned j = 0; j < 4; j++) {
			fprintf(fp, "%f ", _m[i][j]);
		}
		fprintf(fp, "\n");
	}
}


void Matrix4::scan(FILE *fp)
{
#if 0
//	REQUIRE(NULL != fp);
	for (unsigned i = 0; i < 4; i++) {
		for (unsigned j = 0; j < 4; j++) {
			real_t t;
			if (1 != real::scan(fp, &t)) {
				GTB_ERROR("failed to scan matrix");
			}
			_m[i][j] = t;
		}
	}
#endif
}


ostream &operator<<(ostream &out, const Matrix4 &m)
{
	for (unsigned i = 0; i < 4; i++) {
		for (unsigned j = 0; j < 4; j++) {
			out << m[i][j] << " ";
		}
		out << "\n";
	}
	return out;
}


END_GTBMOUSENS
