#include "gtbmouse.h"
#ifndef WIN32
#include <gtb/graphics/point3.hpp>
#include <gtb/graphics/vector3.hpp>
#include <gtb/graphics/line3.hpp>
#endif // WIN32


#ifdef OUTLINE
#define inline
#include <gtb/graphics/point3.ipp>
#undef inline
#endif


using namespace std;


BEGIN_GTBMOUSENS


const Point3 POINT3_ZERO(0.0, 0.0, 0.0);


Point3 &Point3::scale(const Point3 &origin, real_t s)
{
	Vector3 t(origin._p);
	*this -= t;
	_p[0] *= s;
	_p[1] *= s;
	_p[2] *= s;
	*this += t;
	return *this;
}


Point3 &Point3::rotate(const Vector3 &axis, real_t theta)
{
	Vector3 v(_p);
	v.rotate(axis, theta);
	this->reset(v.x(), v.y(), v.z());
	return *this;
}


Point3 &Point3::rotate(const Line3 &l, real_t theta)
{
	Point3 q = l.point(0);
	Vector3 v = *this - q;
	v.rotate(l.direction(), theta);
	*this = q + v;
	return *this;
}


Point3 Point3::centroid(const vector<Point3> &v)
{
	real_t cx = 0.0;
	real_t cy = 0.0;
	real_t cz = 0.0;

	for (unsigned i = 0; i < v.size(); i++) {
		cx += v[i].x() / v.size();
		cy += v[i].y() / v.size();
		cz += v[i].z() / v.size();
	}
	return Point3(cx, cy, cz);
}


END_GTBMOUSENS
