#ifndef GTB_POINT3_INCLUDED
#define GTB_POINT3_INCLUDED

#include "vector"
#include "gtbmousecommon.h"

BEGIN_GTBMOUSENS


class Vector3;
class Line3;
class Matrix4;


class Point3 {
public:
	Point3();
	Point3(const Point3 &p);
	Point3(real_t px, real_t py, real_t pz);
	Point3(const real_t p[3]);
	Point3 &operator=(const Point3 &p);

	real_t x() const;
	real_t y() const;
	real_t z() const;

	void set_x(real_t px);
	void set_y(real_t py);
	void set_z(real_t pz);
	Point3 &reset(real_t px, real_t py, real_t pz);

	real_t operator[](unsigned i) const;
	real_t &operator[](unsigned i);

	bool operator==(const Point3 &p) const;
	bool operator!=(const Point3 &p) const;
	bool is_zero() const;

	Point3 &operator*=(const Matrix4 &m);
	Point3 &operator+=(const Vector3 &v);
	Point3 &operator-=(const Vector3 &v);

	void add(const Point3& p);
	void scalar_scale(double s);
	Point3 scalar_scaled(double s) const;

	friend Point3 operator*(const Matrix4 &m, const Point3 &p);
	friend Point3 operator+(const Point3 &p, const Vector3 &v);
	friend Vector3 operator-(const Point3 &p, const Point3 &q);
	friend Point3 operator-(const Point3 &p, const Vector3 &v);

	Point3 &scale(const Point3 &origin, real_t s);
	Point3 &translate(const Vector3 &v);
	Point3 &translate(real_t dx, real_t dy, real_t dz);
	Point3 &rotate(const Vector3 &axis, real_t theta);
	Point3 &rotate(const Line3 &l, real_t theta);
	Point3 &transform(const Matrix4 &m);
	Point3 &affine_transform(const Matrix4 &m);

	static real_t distance(const Point3 &p, const Point3 &q);
	static real_t squared_distance(const Point3 &p, const Point3 &q);

	static bool collinear(const Point3 &A,
			      const Point3 &B,
			      const Point3 &C);

	static Vector3 normal(const Point3 &A,
			      const Point3 &B,
			      const Point3 &C);

	static Point3 midpoint(const Point3 &A, const Point3 &B);

	static Point3 centroid(const Point3 &A,
			       const Point3 &B,
			       const Point3 &C);
	static Point3 centroid(const Point3 &A,
			       const Point3 &B,
			       const Point3 &C,
			       const Point3 &D);
	static Point3 centroid(const std::vector<Point3> &v);

	void load() const;
	//void render() const;

	void read(FILE *fp);
	void write(FILE *fp) const;

	friend class Vector3;
  	friend std::istream &operator>>(std::istream &s, Point3 &p);
  	friend std::ostream &operator<<(std::ostream &s, const Point3 &p);

protected:
	real_t _p[3];
};


extern const Point3 POINT3_ZERO;


END_GTBMOUSENS

#ifndef OUTLINE
#include "point3.ipp"
#endif

#endif // GTB_POINT3_INCLUDED
