#include "gtbmousecommon.h"
#include "point3.hpp"

#ifndef GTB_VECTOR3_INCLUDED
#define GTB_VECTOR3_INCLUDED


BEGIN_GTBMOUSENS


class Vector3 {
public:
	Vector3();
	Vector3(const Vector3 &v);
	explicit Vector3(const Point3 &p);
	Vector3(real_t vx, real_t vy, real_t vz);
	explicit Vector3(const real_t v[3]);
	explicit Vector3(const real_t v);
	Vector3 &operator=(const Vector3 &v);

	real_t x() const;
	real_t y() const;
	real_t z() const;

	void set_x(real_t vx);
	void set_y(real_t vy);
	void set_z(real_t vz);
	Vector3 &reset(real_t vx, real_t vy, real_t vz);

	real_t operator[](unsigned i) const;
	real_t &operator[](unsigned i);

	bool operator==(const Vector3 &v) const;
	bool operator!=(const Vector3 &v) const;
	bool is_zero() const;

	Vector3 operator-() const;
	Vector3 &operator*=(const Matrix4 &m);
	Vector3 &operator+=(const Vector3 &v);
	Vector3 &operator-=(const Vector3 &v);
	Vector3 &operator*=(real_t a);
	Vector3 &operator/=(real_t a);

	real_t operator*(const Vector3 &v) const;
	real_t operator*(const Point3 &p) const;

	friend Vector3 operator+(const Vector3 &u, const Vector3 &v);
	friend Point3 operator+(const Point3 &p, const Vector3 &v);
	friend Vector3 operator-(const Vector3 &u, const Vector3 &v);
	friend Point3 operator-(const Point3 &p, const Vector3 &v);
	friend Vector3 operator*(real_t a, const Vector3 &v);
	friend Vector3 operator*(const Vector3 &v, real_t a);
	friend Vector3 operator*(const Matrix4 &m, const Vector3 &v);
	friend Vector3 operator/(const Vector3 &v, real_t a);

	real_t dot(const Vector3 &v) const;
	real_t dot(const Point3 &p) const;
	Vector3 cross(const Vector3 &v) const;
	real_t length() const;
	real_t squared_length() const;
	Vector3 &normalize();
	Vector3 normalized() const;
	bool is_normalized() const;
	Vector3 &scale(real_t a);
	Vector3 &rotate(const Vector3 &axis, real_t theta);
	Vector3 &transform(const Matrix4 &m);
	Vector3 &affine_transform(const Matrix4 &m);
	void flip();
	Vector3 flipped() const;

	void load_as_normal() const;

	void read(FILE *fp);
	void write(FILE *fp) const;

	static Matrix4 rotation(const Vector3 &from, const Vector3 &to);

	friend class Point3;
	friend void swap(Vector3 &a, Vector3 &b);
  	friend std::istream &operator>>(std::istream &s, Vector3 &v);
  	friend std::ostream &operator<<(std::ostream &s, const Vector3 &v);

protected:
	real_t _v[3];
};


extern const Vector3 VECTOR3_ZERO;
extern const Vector3 VECTOR3_POSITIVE_X;
extern const Vector3 VECTOR3_NEGATIVE_X;
extern const Vector3 VECTOR3_POSITIVE_Y;
extern const Vector3 VECTOR3_NEGATIVE_Y;
extern const Vector3 VECTOR3_POSITIVE_Z;
extern const Vector3 VECTOR3_NEGATIVE_Z;


END_GTBMOUSENS

#ifndef OUTLINE
#include "vector3.ipp"
#endif

#endif // GTB_VECTOR3_INCLUDED
